package com.agilefaqs.rft.suggestions;

import com.agilefaqs.rft.domain.Country;
import com.agilefaqs.rft.domain.LocationInformation;
import com.agilefaqs.rft.utils.StringUtils;

public class SuggestionParam {
    private final String userFirstName;
    private final String userLastName;
    private final LocationInformation locationInfo;
    private final String validFirstNameToken;
    private final String validLastNameToken;

    public SuggestionParam(LocationInformation locationInfo, String userFirstName, String userLastName) {
        this.locationInfo = locationInfo;
        this.userFirstName = userFirstName;
        this.validFirstNameToken = getToken(userFirstName);
        this.userLastName = userLastName;
        this.validLastNameToken = getToken(userLastName);
    }

    private String getToken(String token) {
        char[] allowedChars = new char[] { '-' };
        String word = StringUtils.removeSpecialCharactersIfAny(token, allowedChars);
        word = appendLettersIfRequired(word);
        word = truncateLettersIfRequired(word);
        return word;
    }

    /**
     * @param word
     * @return
     */
    private String truncateLettersIfRequired(String word) {
        int maxLength = 15;
        if (word.length() > maxLength) {
            word = word.substring(0, maxLength);
        }
        return word;
    }

    /**
     * @param word
     * @return
     */
    private String appendLettersIfRequired(String word) {
        while (word.length() < 3)
            word = word.concat(word);
        return word;
    }

    @Override
    public boolean equals(final Object other) {
        if (this == other) {
            return true;
        }
        if (other == null) {
            return false;
        }
        if (!this.getClass().getName().equals(other.getClass().getName())) {
            return false;
        }
        return equals((SuggestionParam) other);
    }

    private boolean equals(final SuggestionParam other) {
        return userFirstName.equals(other.userFirstName) && userLastName.equals(other.userLastName)
                && getCountryName().equals(other.getCountryName()) && getCity().equals(other.getCity());
    }

    public String getCity() {
        if (locationInfo != null) {
            return locationInfo.getCity();
        }
        return null;
    }

    public String getCountryMoniker() {
        if (locationInfo != null) {
            return locationInfo.getCountryMoniker();
        }
        return null;
    }

    public String getCountryName() {
        if (locationInfo != null) {
            return locationInfo.getCountryName();
        }
        return null;
    }

    public String getValidFirstNameToken() {
        return validFirstNameToken;
    }

    public String getValidLastNameToken() {
        return validLastNameToken;
    }

    public String getUserFirstName() {
        return userFirstName;
    }

    public String getUserLastName() {
        return userLastName;
    }

    public Country getCountry() {
        return locationInfo.getCountry();
    }
}
