package com.agilefaqs.rft.suggestions;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;
import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.List;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.agilefaqs.rft.domain.Country;
import com.agilefaqs.rft.domain.LocationInformation;

public class IdentityGeneratorTest {
    private static final String FIRST_NAME = "Naresh";
    private static final String LAST_NAME = "Jain";
    private static final String AT = "@";
    private final Country country = new Country("IN", "INDIA", "Indian");
    private final SuggestionStrategyFactory suggestionStrategyFactory = createMock(SuggestionStrategyFactory.class);
    private IdentityGenerator IdentityGenerator;
    final private IdentitySuggestionOneStrategy identitySuggestionOneStrategy = createMock(IdentitySuggestionOneStrategy.class);
    final private IdentitySuggestionTwoStrategy identitySuggestionTwoStrategy = createMock(IdentitySuggestionTwoStrategy.class);
    final private IdentitySuggestionThreeStrategy identitySuggestionThreeStrategy = createMock(IdentitySuggestionThreeStrategy.class);
    final private IdentitySuggestionFourStrategy identitySuggestionFourStrategy = createMock(IdentitySuggestionFourStrategy.class);
    final private String suggestion = FIRST_NAME.concat(AT).concat(LAST_NAME).concat(".com");
    private final SuggestionParam params = new SuggestionParam(new LocationInformation(country, "mumbai"), FIRST_NAME, LAST_NAME);

    @Before
    public void setUp() {
        reset(suggestionStrategyFactory);
        final ArrayList<SuggestionStrategy> strategies = new ArrayList<SuggestionStrategy>();
        strategies.add(identitySuggestionOneStrategy);
        strategies.add(identitySuggestionTwoStrategy);
        strategies.add(identitySuggestionThreeStrategy);
        strategies.add(identitySuggestionFourStrategy);
        expect(suggestionStrategyFactory.getStrategies()).andReturn(strategies);
        replay(suggestionStrategyFactory);
        IdentityGenerator = new IdentityGenerator(suggestionStrategyFactory);
        verify(suggestionStrategyFactory);
    }

    @After
    public void verifyMocks() {
        verify(identitySuggestionOneStrategy, identitySuggestionTwoStrategy, identitySuggestionThreeStrategy,
                identitySuggestionFourStrategy);
    }

    private void stopRecordingAndReplay() {
        replay(identitySuggestionOneStrategy, identitySuggestionTwoStrategy, identitySuggestionThreeStrategy,
                identitySuggestionFourStrategy);
    }

    @Test
    public void shouldGenerateFourSuggestionsIfAllStrategiesReturnsAIdentity() throws Exception {
        expect(identitySuggestionOneStrategy.getIdentity(params)).andReturn(suggestion);
        expect(identitySuggestionTwoStrategy.getIdentity(params)).andReturn(suggestion);
        expect(identitySuggestionThreeStrategy.getIdentity(params)).andReturn(suggestion);
        expect(identitySuggestionFourStrategy.getIdentity(params)).andReturn(suggestion);
        stopRecordingAndReplay();
        List<String> ids = IdentityGenerator.getGeneratedIDs(params);
        assertEquals("Expected 4 but was " + ids.size(), 4, ids.size());

    }

    @Test
    public void shouldGenerateFourSuggestionsIfAllStrategiesReturnsNull() throws Exception {
        expect(identitySuggestionOneStrategy.getIdentity(params)).andReturn(null);
        expect(identitySuggestionTwoStrategy.getIdentity(params)).andReturn(null);
        expect(identitySuggestionThreeStrategy.getIdentity(params)).andReturn(null);
        expect(identitySuggestionFourStrategy.getIdentity(params)).andReturn(null);
        stopRecordingAndReplay();
        List<String> ids = IdentityGenerator.getGeneratedIDs(params);
        assertEquals("Expected 0 but was " + ids.size(), 0, ids.size());

    }

    @Test
    public void shouldGenerateTwoSuggestionsIfTwoStrategiesReturnsNull() throws Exception {
        expect(identitySuggestionOneStrategy.getIdentity(params)).andReturn(suggestion);
        expect(identitySuggestionTwoStrategy.getIdentity(params)).andReturn(suggestion);
        expect(identitySuggestionThreeStrategy.getIdentity(params)).andReturn(null);
        expect(identitySuggestionFourStrategy.getIdentity(params)).andReturn(null);
        stopRecordingAndReplay();
        List<String> ids = IdentityGenerator.getGeneratedIDs(params);
        assertEquals("Expected 2 but was " + ids.size(), 2, ids.size());

    }

}
