package com.agilefaqs.rft.suggestions.strategies;

import static org.easymock.classextension.EasyMock.createMock;
import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import com.agilefaqs.rft.domain.UserService;
import com.agilefaqs.rft.suggestions.RandomNumberGenerator;
import com.agilefaqs.rft.suggestions.SuggestionsUtil;
import com.agilefaqs.rft.suggestions.strategies.ChildSuggestionFactory;
import com.agilefaqs.rft.suggestions.strategies.ChildSuggestionStrategy;
import com.agilefaqs.rft.suggestions.strategies.FirstNameAppendLastNameStrategy;
import com.agilefaqs.rft.suggestions.strategies.LastNameInitialStrategy;
import com.agilefaqs.rft.suggestions.strategies.LastNameInitialYearBasedStrategy;
import com.agilefaqs.rft.suggestions.strategies.RandomNumberBasedStrategy;
import com.agilefaqs.rft.suggestions.strategies.SimpleTokenStrategy;
import com.agilefaqs.rft.suggestions.strategies.YearBasedStrategy;

public class ChildSuggestionFactoryTest {
    private final RandomNumberGenerator randomNumberGenerator = createMock(RandomNumberGenerator.class);
    private final UserService userService = createMock(UserService.class);
    private final SuggestionsUtil utils = createMock(SuggestionsUtil.class);
    private final List<ChildSuggestionStrategy> strategies = new ArrayList<ChildSuggestionStrategy>();
    private final List<ChildSuggestionStrategy> strategiesForSuggestionOne = new ArrayList<ChildSuggestionStrategy>();

    @Test
    public void shouldGetAllTheStrategies() throws Exception {
        final SimpleTokenStrategy simpleTokenStrategy = new SimpleTokenStrategy(userService, utils);
        final YearBasedStrategy yearBasedStrategy = new YearBasedStrategy(userService, utils);
        final RandomNumberBasedStrategy randomNumberBasedStrategy = new RandomNumberBasedStrategy(userService, utils,
                randomNumberGenerator);
        strategies.add(simpleTokenStrategy);
        strategies.add(new FirstNameAppendLastNameStrategy(userService, utils));
        strategies.add(new LastNameInitialStrategy(userService, utils));
        strategies.add(yearBasedStrategy);
        strategies.add(new LastNameInitialYearBasedStrategy(userService, utils));
        strategies.add(randomNumberBasedStrategy);

        strategiesForSuggestionOne.add(simpleTokenStrategy);
        strategiesForSuggestionOne.add(yearBasedStrategy);
        strategiesForSuggestionOne.add(randomNumberBasedStrategy);
        ChildSuggestionFactory factory = new ChildSuggestionFactory(userService, utils, randomNumberGenerator);
        final List<ChildSuggestionStrategy> allStrategies = factory.getStrategies();
        assertEquals("expected was 6 but was : " + allStrategies.size(), 6, allStrategies.size());

        final List<ChildSuggestionStrategy> childStrategies = factory.getChildStrategiesForSuggestionOne();
        assertEquals("expected was 3 but was : " + childStrategies.size(), 3, childStrategies.size());
    }

}
