package com.agilefaqs.rft.suggestions.strategies;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.agilefaqs.rft.domain.UserService;
import com.agilefaqs.rft.suggestions.IDTokens;
import com.agilefaqs.rft.suggestions.SuggestionsUtil;

public class LastNameInitialStrategyTest {

    private static final String COUNTRY_MONIKER = "Indian";
    private static final String FIRST_NAME = "Naresh";
    private static final String LAST_NAME = "Jain";
    private static final String LAST_NAME_INTIAL = String.valueOf(LAST_NAME.charAt(0));
    private final ChildStrategyParam childStrategyParam = new ChildStrategyParam(FIRST_NAME, COUNTRY_MONIKER, LAST_NAME);

    private final String Identity = FIRST_NAME.concat(LAST_NAME_INTIAL).concat("@").concat(COUNTRY_MONIKER).concat(".com");
    private final SuggestionsUtil suggestionUtil = createMock(SuggestionsUtil.class);
    private final UserService userService = createMock(UserService.class);
    private final LastNameInitialStrategy lastNameInitialYearStrategy = new LastNameInitialStrategy(userService, suggestionUtil);

    @Before
    public void setUp() {
        reset(userService, suggestionUtil);
    }

    @Test
    public void shouldGenerateSuggestionBasedOnLastNameInitialStratey() throws Exception {
        final String token1 = childStrategyParam.getToken1().concat(LAST_NAME_INTIAL);
        final String token2 = childStrategyParam.getToken2();
        expect(suggestionUtil.getIdentityTokens(token1, token2)).andReturn(new IDTokens(token1, token2));
        expect(suggestionUtil.appendTokensForId(token1, token2)).andReturn(Identity);
        expect(userService.isIdentityAvailable(Identity)).andReturn(true);
        stopRecordingAndStartReplay();
        String Id = lastNameInitialYearStrategy.generateId(childStrategyParam);
        assertEquals(Id, Identity);
    }

    @Test
    public void shouldReturnNullIfIdIsNotAvailable() throws Exception {
        final String token1 = childStrategyParam.getToken1().concat(LAST_NAME_INTIAL);
        final String token2 = childStrategyParam.getToken2();
        expect(suggestionUtil.getIdentityTokens(token1, token2)).andReturn(null);
        stopRecordingAndStartReplay();
        String generatedId = lastNameInitialYearStrategy.generateId(childStrategyParam);
        assertNull(generatedId);
    }

    @Test
    public void shouldReturnNullIfIdTokenReturnedBySuggestionUtilIsNullBasedOnYearStratey() throws Exception {
        final String token1 = childStrategyParam.getToken1().concat(LAST_NAME_INTIAL);
        final String token2 = childStrategyParam.getToken2();
        expect(suggestionUtil.getIdentityTokens(token1, token2)).andReturn(new IDTokens(token1, token2));
        expect(suggestionUtil.appendTokensForId(token1, token2)).andReturn(Identity);
        expect(userService.isIdentityAvailable(Identity)).andReturn(false);
        stopRecordingAndStartReplay();
        String generatedId = lastNameInitialYearStrategy.generateId(childStrategyParam);
        assertNull(generatedId);
    }

    @After
    public void verifyMocks() {
        verify(userService, suggestionUtil);
    }

    private void stopRecordingAndStartReplay() {
        replay(suggestionUtil, userService);
    }

}
