package com.agilefaqs.rft.suggestions.strategies;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.agilefaqs.rft.domain.UserService;
import com.agilefaqs.rft.suggestions.IDTokens;
import com.agilefaqs.rft.suggestions.RandomNumberGenerator;
import com.agilefaqs.rft.suggestions.SuggestionsUtil;
import com.agilefaqs.rft.suggestions.strategies.ChildStrategyParam;
import com.agilefaqs.rft.suggestions.strategies.RandomNumberBasedStrategy;

public class RandomNumberBasedStrategyTest {

    private static final String COUNTRY_MONIKER = "indian";
    private static final String FIRST_NAME = "Naresh";
    private static final String RANDOM = "100000";
    private static final String RANDOM1 = "1999";
    private static final String RANDOM2 = "668678";
    private static final String RANDOM3 = "8989";
    private static final String RANDOM4 = "333";
    private final String Identity3 = FIRST_NAME.concat(RANDOM3).concat("@").concat(COUNTRY_MONIKER).concat(".com");

    private final ChildStrategyParam childStrategyParam = new ChildStrategyParam(FIRST_NAME, COUNTRY_MONIKER, null);
    private final String Identity = FIRST_NAME.concat(RANDOM).concat("@").concat(COUNTRY_MONIKER).concat(".com");
    private final String Identity1 = FIRST_NAME.concat(RANDOM1).concat("@").concat(COUNTRY_MONIKER).concat(".com");
    private final String Identity2 = FIRST_NAME.concat(RANDOM2).concat("@").concat(COUNTRY_MONIKER).concat(".com");
    private final String Identity4 = FIRST_NAME.concat(RANDOM4).concat("@").concat(COUNTRY_MONIKER).concat(".com");
    private final UserService userService = createMock(UserService.class);
    private final RandomNumberGenerator randomNumberGenerator = createMock(RandomNumberGenerator.class);
    private final SuggestionsUtil suggestionUtil = createMock(SuggestionsUtil.class);
    private final RandomNumberBasedStrategy randomNumberBasedStrategy = new RandomNumberBasedStrategy(userService, suggestionUtil,
            randomNumberGenerator);

    @Before
    public void setUp() {

        reset(userService, suggestionUtil, randomNumberGenerator);
    }

    @Test
    public void shouldGenerateSuggestionBasedOnLastNameIntialAndYearStratey() throws Exception {
        final String token1 = childStrategyParam.getToken1();
        final String token2 = childStrategyParam.getToken2();
        expect(suggestionUtil.getIdentityTokens(token1, token2)).andReturn(new IDTokens(token1, token2));
        expect(randomNumberGenerator.next()).andReturn(RANDOM);
        expect(suggestionUtil.appendTokensForId(token1.concat(RANDOM), token2)).andReturn(Identity);
        expect(userService.isIdentityAvailable(Identity)).andReturn(true);
        stopRecordingAndStartReplay();
        String Id = randomNumberBasedStrategy.generateId(childStrategyParam);
        assertEquals(Id, Identity);
    }

    @Test
    public void shouldReturnNullIfIdIsNotAvailable() throws Exception {
        final String token1 = childStrategyParam.getToken1();
        final String token2 = childStrategyParam.getToken2();
        expect(suggestionUtil.getIdentityTokens(token1, token2)).andReturn(null);
        stopRecordingAndStartReplay();
        String generatedId = randomNumberBasedStrategy.generateId(childStrategyParam);
        assertNull(generatedId);
    }

    @Test
    public void shouldReturnNullIfIdIfMoreThanFiveAttempts() throws Exception {
        final String token1 = childStrategyParam.getToken1();
        final String token2 = childStrategyParam.getToken2();
        expect(suggestionUtil.getIdentityTokens(token1, token2)).andReturn(new IDTokens(token1, token2));
        expect(randomNumberGenerator.next()).andReturn(RANDOM);
        expect(randomNumberGenerator.next()).andReturn(RANDOM1);
        expect(randomNumberGenerator.next()).andReturn(RANDOM2);
        expect(randomNumberGenerator.next()).andReturn(RANDOM3);
        expect(randomNumberGenerator.next()).andReturn(RANDOM4);
        expect(suggestionUtil.appendTokensForId(token1.concat(RANDOM), token2)).andReturn(Identity);
        expect(suggestionUtil.appendTokensForId(token1.concat(RANDOM1), token2)).andReturn(Identity1);
        expect(suggestionUtil.appendTokensForId(token1.concat(RANDOM2), token2)).andReturn(Identity2);
        expect(suggestionUtil.appendTokensForId(token1.concat(RANDOM3), token2)).andReturn(Identity3);
        expect(suggestionUtil.appendTokensForId(token1.concat(RANDOM4), token2)).andReturn(Identity4);
        expect(userService.isIdentityAvailable(Identity)).andReturn(false);
        expect(userService.isIdentityAvailable(Identity1)).andReturn(false);
        expect(userService.isIdentityAvailable(Identity2)).andReturn(false);
        expect(userService.isIdentityAvailable(Identity3)).andReturn(false);
        expect(userService.isIdentityAvailable(Identity4)).andReturn(false);
        stopRecordingAndStartReplay();
        String Id = randomNumberBasedStrategy.generateId(childStrategyParam);
        assertNull(Id);
    }

    @After
    public void verifyMocks() {
        verify(userService, suggestionUtil, randomNumberGenerator);
    }

    private void stopRecordingAndStartReplay() {
        replay(suggestionUtil, userService, randomNumberGenerator);
    }

}
